#ifndef __AboutDialog__
#define __AboutDialog__

// $Id: AboutDialog.h 14011 2009-08-06 11:34:04Z ritt $

/**
@file
Subclass of AboutDialog_fb, which is generated by wxFormBuilder.
*/

/** Implementing ConfigDialog_fb */
class AboutDialog : public AboutDialog_fb
{
public:
   /** Constructor */
   AboutDialog( wxWindow* parent );
};

#endif // __AboutDialog__
