#ifndef __InfoDialog__
#define __InfoDialog__

// $Id: InfoDialog.h 15243 2010-05-07 14:01:43Z ritt $

/**
@file
Subclass of InfoDialog_fb, which is generated by wxFormBuilder.
*/

/** Implementing ConfigDialog_fb */
class InfoDialog : public InfoDialog_fb
{
public:
   /** Constructor */
   InfoDialog( wxWindow* parent );
};

#endif // __InfoDialog__
