#ifndef __DisplayDialog__
#define __DisplayDialog__

// $Id: DisplayDialog.h 21252 2014-02-06 09:37:27Z ritt $

/**
@file
Subclass of DisplayDialog_fb, which is generated by wxFormBuilder.
*/

class DOFrame;
class Osci;

/** Implementing DisplayDialog_fb */
class DisplayDialog : public DisplayDialog_fb
{
protected:
   // Handlers for DisplayDialog_fb events.
   void OnDateTime(wxCommandEvent& event);
   void OnShowGrid(wxCommandEvent& event);
   void OnLines(wxCommandEvent& event);
   void OnDisplayMode(wxCommandEvent& event);
   void OnButton(wxCommandEvent& event);
   void OnScalers(wxCommandEvent& event);
                 
   void OnClose(wxCommandEvent& event);
   
public:
   /** Constructor */
   DisplayDialog( wxWindow* parent );

private:
   DOFrame *m_frame;
   Osci    *m_osci;

   void PopulateBoards(void);
   void EnableButtons(void);

};

#endif // __DisplayDialog__
